# vim: ts=4
###
#
# Listen is the legal property of mehdi abaakouk <theli48@gmail.com>
# Copyright (c) 2006 Mehdi Abaakouk
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
###


"""
gnomevfswrap is a wrapper library for builtin function file and open
when python use this function and filename is a uri 
gnomevfs is used instead of builtin function
not all behavior are implemented but is sufficient for mutagen work with gnomevfs support

note: when gnomevfs is used, mmap is disable in mutagen, this is cause slow down tag writting

"""


import __builtin__
import gnomevfs
import traceback

DEBUG = True   

class WrapFile(gnomevfs.Handle):
        
    def __init__(self,filename,option,perm=None):

        self.filename = filename
        self.realoption = self.parse_option(option)
        
        print "file(",filename,",",option,",",perm,"):",self.realoption
        try:
            gnomevfs.Handle.__init__(self,filename,self.realoption)
        except:
            if DEBUG :print traceback.format_exc()
            raise IOError

    
    def parse_option(self,options):
        options = options.replace("b","")
        
        
        realoptions = {
            "r":gnomevfs.OPEN_READ ,
            "r+":gnomevfs.OPEN_READ | gnomevfs.OPEN_WRITE | gnomevfs.OPEN_RANDOM ,
            "w":gnomevfs.OPEN_READ | gnomevfs.OPEN_WRITE ,
            "w+":gnomevfs.OPEN_READ | gnomevfs.OPEN_WRITE | gnomevfs.OPEN_RANDOM  ,
            "a":gnomevfs.OPEN_READ | gnomevfs.OPEN_WRITE
        }
        for args, realoption in realoptions.iteritems():
            if len([a for a in args.split() if a not in options.split()])==0:
                return realoption
        else:
            raise IOError ("Unknown option:", options)
    
    def read(self,nbytes=None):
        try:
            data = ''
            if nbytes!=None:
                nrealbytes = 0
                while nbytes!=nrealbytes:
                    try:data += gnomevfs.Handle.read(self,nbytes-nrealbytes)
                    except gnomevfs.EOFError:
                        break
                    nrealbytes += len(data)
                return data
            else:
                buffer = 1024
                while True:
                    try: data += gnomevfs.Handle.read(self,buffer)
                    except gnomevfs.EOFError:
                        break
                return data
        except:
            if DEBUG: print traceback.format_exc()
            raise IOError

    def seek(self,offset,whence=0):
        if whence==0: whence = gnomevfs.SEEK_START
        elif whence==1: whence = gnomevfs.SEEK_CURRENT
        elif whence==2: whence = gnomevfs.SEEK_END
        try:
            return gnomevfs.Handle.seek(self,offset,whence)
        except gnomevfs.BadParametersError:
            raise IOError
        except:
            if DEBUG: print traceback.format_exc()
            raise IOError

    def write(self,data):
        try:
            return gnomevfs.Handle.write(self,data)
        except:
            if DEBUG: print traceback.format_exc()
            raise IOError
            
    def flush(self):
        pass

    def truncate(self,length=None):
        try:
            gnomevfs.Handle.truncate(self,length)
        except:
            if DEBUG: print traceback.format_exc()
            raise IOError
        
        
    def fileno(self):
        """ 
        Sad, but gnomevfs doesn't have filedescriptor
        Then with gnomevfs, mutagen can't use mmap
        """
        raise EnvironmentError
        

def WrapGetsize(filename):
    try:
        fileinfo = gnomevfs.get_file_info(filename)
        return fileinfo.size
    except:
        return 0

import os.path
builtin_open = __builtin__.open
builtin_file = __builtin__.file
builtin_getsize = os.path.getsize

def file(*args,**kargs):
    if (args[0] and args[0].find("://")!=-1) or (kargs.has_key("filename") and kargs.get("filename").find("://")!=-1):
        return WrapFile(*args,**kargs)
    else:
        return builtin_file(*args,**kargs)

def open(*args,**kargs):
    if (args[0] and args[0].find("://")!=-1) or (kargs.has_key("filename") and kargs.get("filename").find("://")!=-1):
        return WrapFile(*args,**kargs)
    else:
        return builtin_open(*args,**kargs)
    
def getsize(filename):
    if (filename and filename.find("://")!=-1):
        return WrapGetsize(filename)
    else:
        return builtin_getsize(filename)

os.path.getsize = getsize
__builtin__.open = open
__builtin__.file = file
